/*
 * Copyright:      Fabian Maximilian Thiele  mailto:me@apetech.de
 * Author:         Fabian Maximilian Thiele
 * Remarks:        this Copyright must be included
 * known Problems: none
 * Version:        1.1
 * Description:    KS0108 Library Demo Program 
 * 
 */

#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/delay.h>
#include <string.h>
#include <stdlib.h>

#include "ks0108.h"
#define N		32
#define M		8
#define SIZE	8

unsigned char mat[2][N][M];

int isAlive(int i, int j, int index){
	int y = j / SIZE, iy = j % SIZE;
	return ((mat[index][i][y] & (128 >> iy)) > 0);
}

void setCell(int i, int j, int index){
	int y = j / SIZE, iy = j % SIZE;
	mat[index][i][y] |= (128 >> iy);
}

int getNeigh(int i, int j, int index){
	int count = 0;
	int y, iy;
	int xx[8] = {i - 1, i - 1, i - 1, i, i, i + 1, i + 1, i + 1};
	int yy[8] = {j - 1, j, j + 1, j - 1, j + 1, j - 1, j, j + 1};
	int k;
	for(k = 0; k < 8; k++){
		if(xx[k] >= 0 && xx[k] < N && yy[k] >= 0 && yy[k] < M * SIZE){
			y = yy[k] / SIZE;
			iy = yy[k] % SIZE;
			if((mat[index][xx[k]][y] & (128 >> iy)) > 0)
				count++;
		}
	}	
	
	return count;
}


int main(void) {
	
	MCUCSR |=(1<<JTD);
	MCUCSR |=(1<<JTD); 
	
	_delay_ms(1);
	// Wait a little while the display starts up
	for(volatile uint16_t i=0; i<15000; i++);
	
	// Initialize the LCD
	ks0108Init(0);
	/*

	while(1){
		// a nice little round rect
		ks0108DrawRoundRect(5, 5, 117, 20, 8, BLACK);
		_delay_ms(2000);
		ks0108DrawRoundRect(5, 5, 117, 20, 8, WHITE);
		_delay_ms(2000);

	}
*/
	int i, j, index = 0;
	srand(0);
	for(i = 0; i < N; i++)
		for(j = 0; j < M; j++){
				mat[index][i][j] = rand() % 256;
			}



	int count, change = 1;
	while(change){
		//afiseaza matricea
		for(i = 0; i < N; i++)
			for(j = 0; j < 64; j += 8){
				ks0108GotoXY(i, j);
				ks0108WriteData(mat[index][i][j/8]);
		}
		
		
		
		memset(mat[1 - index], 0, N * M);
		change = 0;
		for(i = 0; i < N; i++)
			for(j = 0; j < M * SIZE; j++){
				count = getNeigh(i, j, index);
				if(count == 2 && isAlive(i, j, index)){
					setCell(i, j, 1 - index);
					change = 1;
				}
				else if(count == 3){
					setCell(i, j, 1 - index);
					change = 1;
				}
			}

		
		index = 1 - index;
		_delay_ms(1000);
		
	}

}
